@@ -1,9 +1,10 @@
 """
 Factorization of integers:
     factorint(n) factorize the integer n into prime factors
+    divisors(n) unsorted list of all divisors of the integer n
 """
 
-from math import isqrt, gcd
+from math import isqrt, gcd, prod
 from .primes import sieve_eratosthenes, is_prime
 from .factor_pm1 import _pm1_parameters, factor_pm1
 from .factor_ecm import _ecm_parameters, factor_ecm
@@ -13,7 +14,6 @@ from .factor_ecm import _ecm_parameters, factor_ecm
 # Factoring
 
 
-
 def _factor_fermat(n: int, steps: int = 10) -> list:
     "Fermat method"
     parameters = {11: (12, 6), 23: (12, 0),
@@ -28,9 +28,16 @@ def _factor_fermat(n: int, steps: int = 10) -> list:
         if b * b == a * a - n:
             return a - b
 
-def factorint(n: int, verbose: int = 0) -> list:
-    "Factor a number."
+def factorint(n: int, verbose: int = 0) -> dict:
+    "Factor an ineger `n` into prime factors."
     prime_factors = {}
+    if not isinstance(n, int):
+        raise ValueError("Number to be factored must be an integer!")
+    if n == 0:
+        return {0: 1}
+    if n < 0:
+        n *= -1
+        prime_factors[-1] = 1
 
     def add_factors(m: int, mm: tuple) -> None:
         k = remaining_factors[m]
@@ -88,7 +95,7 @@ def factorint(n: int, verbose: int = 0) -> list:
         D = isqrt(B2)
         primes = sieve_eratosthenes(B1 - 1 + ((B2 - B1 + 1) // (2 * D) + 1) * 2 * D)
         pm1_parameters = _pm1_parameters(10 * B1, B2, primes = primes)
-        ecm_parameters = tuple([num_curves] + list(_ecm_parameters(B1, B2, D, primes = primes)))
+        ecm_parameters = tuple([B1, B2, num_curves] + list(_ecm_parameters(B1, B2, D, primes = primes)))
 
         methods = {_factor_fermat: "fm", factor_pm1: "pm1", factor_ecm: "ecm"}  #, factor_qs: "qs"}
         while remaining_factors:
@@ -96,7 +103,8 @@ def factorint(n: int, verbose: int = 0) -> list:
             for method in [ _factor_fermat, factor_pm1, factor_ecm ]:  # , factor_qs ]:
                 factors = list(remaining_factors)
                 for m in factors:
-                    if verbose > 1: print("Factoring: ",m, "method", methods[method])
+                    if verbose > 1:
+                        print("Factoring: ",m, "method", methods[method])
                     if method == factor_pm1:
                         tmp = factor_pm1(m, pm1_parameters = pm1_parameters)
                     elif method == factor_ecm:
@@ -131,5 +139,35 @@ def factorint(n: int, verbose: int = 0) -> list:
         if len(remaining_factors) == 0:
             return prime_factors
 
-    print("Incomplete factorization!")
-    return prime_factors, remaining_factors, new_factors
+    raise ValueError("Incomplete factorization:", prime_factors, remaining_factors, new_factors)
+
+# Divisors
+
+def divisors(n:int, proper:bool = False) -> int:
+    """Returns an unsorted list of all divisors of an integer `n`."""
+    facctordict= factorint(n)
+    primes = [p for p in facctordict]
+    nprimes = len(primes)
+    multiplicites = [facctordict[p] for p in primes]
+
+    exponents = [0] * nprimes
+    if proper:
+        divisors = []
+    else:
+        divisors = [1]
+    i = 0
+    while True:
+        exponents[i] += 1
+        if exponents[i] > multiplicites[i]:
+            while exponents[i] > multiplicites[i]:
+                exponents[i] = 0
+                i += 1
+                exponents[i] += 1
+            i = 0
+        d = prod(primes[i] ** exponents[i] for i in range(nprimes))
+        if d == n:
+            if proper:
+                return divisors
+            divisors.append(n)
+            return divisors
+        divisors.append(d)
