@@ -99,12 +99,12 @@ class Matrix:
         return self.matrix == other.matrix
 
     def map(self, func):
-        "Apply a function to all elements in place"
+        "Apply a function to all elements in place."
         for row in self.matrix:
             row[:] = map(func, row)
 
     def applyfunc(self, func):
-        "Apply a function to all elements"
+        "Apply a function to all elements."
         tmp = self[:,:]
         tmp.map(func)
         return tmp
@@ -169,6 +169,9 @@ class Matrix:
     def __neg__(self) -> "Matrix":
         return -1 * self
 
+    def __pos__(self) -> "Matrix":
+        return self
+
     def __mul__(self, other) -> "Matrix":
         if isinstance(other, Matrix):
             return self.multiply(other)
@@ -208,7 +211,7 @@ class Matrix:
 
     def det(self) -> int:
         "Compute the determinant of a matrix M."
-        if self.rows != self.rows:
+        if self.rows != self.cols:
             raise ValueError("Matrix must be square!")
         n, m = self.cols, self.rows
         R = self[:, :]
@@ -236,7 +239,7 @@ class Matrix:
 
     def inv(self) -> "Matrix":
         "Compute the inverse of a square matrix M."
-        if self.rows != self.rows:
+        if self.rows != self.cols:
             raise ValueError("Matrix must be square!")
         n = self.cols
         MM = Matrix([[0 for _ in range(2*n)] for _ in range(n)])
@@ -250,7 +253,7 @@ class Matrix:
 
     def is_unimodular(self) -> bool:
         "Test if the matrix is unimodular."
-        if self.rows != self.rows:
+        if self.rows != self.cols:
             return False
         def is_integer(i):
             if isinstance(i, int) or (isinstance(i, Fraction) and i.denominator == 1):
@@ -259,7 +262,7 @@ class Matrix:
         return all([is_integer(i) for i in self]) and self.det()**2 == 1
 
     def zeros(self, m: int = None, n: int = None):
-        "Returns a zero matrix of the same dimension"
+        "Returns a zero matrix of the same dimension."
         if not m and not n:
             n, m = self.cols, self.rows
         elif not n:
@@ -268,10 +271,10 @@ class Matrix:
             zero = 0 * self[0]
         except:
             zero = 0
-        return Matrix([[ zero for j in range(n)] for i in range(m) ])
+        return Matrix([[ zero for j in range(n) ] for i in range(m) ])
 
     def eye(self, m: int = None, n: int = None):
-        "Returns an identity matrix of the same dimension"
+        "Returns an identity matrix of the same dimension."
         def delta(i, j):
             if i == j:
                 return one
@@ -288,14 +291,14 @@ class Matrix:
         return Matrix([[ delta(i, j) for j in range(n) ] for i in range(m) ])
 
 
-def zeros(m: int, n: int = None) -> "Matrix":
-    "Returns a zero matrix of the given dimension"
+def zeros(m: int, n: int = None, zero = 0) -> "Matrix":
+    "Returns a zero matrix of the given dimension."
     if not n:
         n = m
-    return Matrix([[ 0 for j in range(n)] for i in range(m) ])
+    return Matrix([[ zero for j in range(n) ] for i in range(m) ])
 
 def eye(m:int, n: int = None) -> "Matrix":
-    "Returns an identity matrix of the given dimension"
+    "Returns an identity matrix of the given dimension."
     def delta(i, j):
         if i == j:
             return 1
