@@ -1,13 +1,16 @@
 """
 Number theory tools:
     egcd(a,b) extended Euclidean agorithm
-    crt([a1, a2, ...],[m1, m2, ...]) Chinese Remainder Theorem
     cf(Fraction(m,n)) continued fraction expansions
     convergents() convergents of a continued fraction
     legendre_symbol(a, p) compute the Legendre symbol of a with respect to the prime p
     jacobi_symbol(a, n) compute the Jacobi symbol of a with respect to n
     sqrt_mod(n, p) square root of n modulo a prime p
+    euler_phi(n) Euler phi function of n
+    carmichael_lambda(n) Carmichael lambda function of n
+    moebius_mu(n) Moebius function of n
     order(a, n) oder of a in the multiplicative group Z_n^*
+    crt([a1, a2, ...],[m1, m2, ...]) Chinese Remainder Theorem
 """
 from math import gcd, lcm, prod
 from fractions import Fraction
@@ -26,83 +29,8 @@ def egcd(a: int, b: int) -> (int, int, int):
     return r0, x0, y0
 
 
-# Chinese remainder theorem
-
-def crt(a: list[int], m: list[int]) -> int:
-    """Solve given linear congruences x % m[j] == a[j] using the Chinese Remainder Theorem."""
-    l = len(a)
-    assert len(m) == l, "The lists of numbers and modules must have equal length."
-    M = prod(m)
-    Mi = [M // m[i] for i in range(l)]
-    MNi = [Mi[i] * pow(Mi[i], -1, m[i]) % M for i in range(l)]
-    return sum([a[i] * MNi[i] % M for i in range(l)]) % M
-
 # Continued fractions
 
-# class Fraction:
-#     "Rationl number"
-#
-#     def __init__(self, numerator: int, denominator: int = 1):
-#         assert denominator, "Denominator must be nozero."
-#         tmp = gcd(denominator, numerator)
-#         if tmp != 1:
-#             denominator //= tmp
-#             numerator //= tmp
-#         self.numerator = numerator
-#         self.denominator = denominator
-#
-#     def __repr__(self):
-#         if self.denominator == 1:
-#             return str(self.numerator)
-#         else:
-#             return str(self.numerator) + "/" + str(self.denominator)
-#
-#     def __eq__(self, other):
-#         if not isinstance(other, self.__class__):
-#             return False
-#         return self.denominator == other.denominator and self.numerator == other.numerator
-#
-#     def __bool__(self):
-#         return self.numerator != 0
-#
-#     def __add__(self, other: "Fraction") -> "Fraction":
-#         if isinstance(other, self.__class__):
-#             return Rational(self.numerator * other.denominator + other.numerator * self.denominator, self.denominator * other.denominator)
-#         if isinstance(other, int):
-#             return Rational(self.numerator + other * self.denominator, self.denominator)
-#         raise ValueError(f"Cannot add {self} and {other}.")
-#
-#     def __neg__(self) -> "Fraction":
-#         return Rational(- self.numerator, self.denominator)
-#
-#     def __sub__(self, other: "Fraction") -> "Fraction":
-#         if isinstance(other, self.__class__):
-#             return Rational(self.numerator * other.denominator - other.numerator * self.denominator, self.denominator * other.denominator)
-#         if isinstance(other, int):
-#             return Rational(self.numerator - other * self.denominator, self.denominator)
-#         raise ValueError(f"Cannot subtract {self} and {other}.")
-#
-#     def __mul__(self, other: "Fraction") -> "Fraction":
-#         if isinstance(other, self.__class__):
-#             return Rational(self.numerator * other.numerator, self.denominator * other.denominator)
-#         if isinstance(other, int):
-#             return Rational(self.numerator * other, self.denominator)
-#         raise ValueError(f"Cannot multiply {self} and {other}.")
-#
-#     def __rmul__(self, other: int) -> "Fraction":
-#         return Rational(self.denominator * other, self.numerator)
-#
-#     def __truediv__(self, other: "Fraction") -> "Fraction":
-#         if isinstance(other, self.__class__):
-#             return Rational(self.numerator * other.denominator, self.denominator * other.numerator)
-#         if isinstance(other, int):
-#             return Rational(self.numerator, self.denominator * other)
-#         raise ValueError(f"Cannot divide {self} and {other}.")
-#
-#     def __pow__(self, scalar: int) -> "Fraction":
-#         return Rational(self.numerator**scalar, self.denominator**scalar)
-
-
 def fraction_repr(self):
     "Representation of a fraction."
     if self.denominator == 1:
@@ -113,7 +41,7 @@ Fraction.__repr__ = fraction_repr
 
 
 def cf(x: Fraction) -> list[int]:
-    "Compute the continued fraction expansion of the rational number m/n."
+    "Compute the continued fraction expansion of the rational fraction `x`."
     res = []
     m = x.numerator
     n = x.denominator
@@ -144,7 +72,7 @@ def convergents(cont_frac: list[int]) -> float:
 # Legendre symbol
 
 def legendre_symbol(a: int, p: int) -> int:
-    """Compute the Legendre symbol of a with respect to the prime p."""
+    """Compute the Legendre symbol of `a` with respect to the prime `p`."""
     a = a % p
     if a == 0:
         return 0
@@ -153,7 +81,7 @@ def legendre_symbol(a: int, p: int) -> int:
     return -1
 
 def jacobi_symbol(a: int, n: int) -> int:
-    """Compute the Jacobi symbol of a with respect to the integer n."""
+    """Compute the Jacobi symbol of `a` with respect to the integer `n`."""
     # Crandall/Pomerance Algorithm 2.3.5
     a = a % n
     t = 1
@@ -172,7 +100,7 @@ def jacobi_symbol(a: int, n: int) -> int:
     return 0
 
 def sqrt_mod(a: int, p: int) -> int:
-    "Compute a square root of `a` modulo `p` unsing Cipolla's algorithm."
+    """Compute a square root of `a` modulo `p` unsing Cipolla's algorithm."""
     a %= p
     if a == 0 or a == 1:
         return a
@@ -198,19 +126,23 @@ def sqrt_mod(a: int, p: int) -> int:
         i = i >> 1  # i= i/2
     return y1
 
-# Euler phi and Carmichael function
+# Euler phi, Carmichael function, Moebius function
 
 from .factor import factorint
 
 
 def euler_phi(n: int) -> int:
     """Euler's phi function of `n`."""
+    if not isinstance(n, int) or n < 1:
+        raise ValueError(f"{n} is not a positive integer")
     k = factorint(n)
     return prod([(p - 1) * p ** (k[p] - 1) for p in k])
 
 
 def carmichael_lambda(n: int) -> int:
     """Carmichael's lambda function of `n`."""
+    if not isinstance(n, int) or n < 1:
+        raise ValueError(f"{n} is not a positive integer")
     k = factorint(n)
     lam_all = []  # values corresponding to the prime factors
     for p in k:
@@ -223,12 +155,26 @@ def carmichael_lambda(n: int) -> int:
         lam = lcm(lam, l)
     return lam
 
+def moebius_mu(n: int) -> int:
+    """Moebius' mu function of `n`."""
+    if not isinstance(n, int) or n < 0:
+        raise ValueError(f"{n} is not a nonegative integer")
+    if n == 0:
+        return 0
+    factors = factorint(n)
+    if any(i > 1 for i in factors.values()):
+        return 0
+    if len(factors) % 2 == 1:
+        return -1
+    return 1
+
 # Order in Z_p^*
 
 def order(a: int, n: int, factor=False) -> int:
     """Compute the order of `a` in the group Z_n^*."""
     a %= n
-    assert a != 0 and gcd(a, n) == 1, f"{a} and {n} are not coprime!"
+    if a == 0 or gcd(a, n) != 1:
+        raise ValueError(f"{a} and {n} are not coprime!")
     factors = {}  # We compute euler_phi(n) and its factorization in one pass
     for p, k in factorint(n).items():  # first factorize n
         for pm, km in factorint(p - 1).items():  # factor p-1 and add the factors
@@ -260,3 +206,40 @@ def order(a: int, n: int, factor=False) -> int:
     if factor:
         return order_a, factors_order
     return order_a
+
+# Chinese remainder theorem
+
+def crt(a: list[int], m: list[int], coprime = True) -> int:
+    """Solve given linear congruences `x % m[j] == a[j]` using the Chinese Remainder Theorem."""
+    l = len(m)
+    if l != len(a):
+        raise ValueError("The lists of numbers and modules must have equal length.")
+
+    if not coprime: # make a copy
+        m = [ mi for mi in m ]
+        a = [ ai for ai in a ]
+        for i in range(l):
+            for j in range(i + 1,l):
+                g = gcd(m[i], m[j])
+                if g == 1: # moduli are coprime, nothing to be done
+                    continue
+                if (a[i] - a[j]) % g:
+                     raise ValueError(f"Congruences not solvable!")
+                for p, k in factorint(g).items():
+                    p = p**k #  remove this factor from one of the equations
+                    if gcd(m[i] // p, p) == 1:
+                        m[i] //= p
+                        a[i] %= m[i]
+                    else:
+                        m[j] //= p
+                        a[j] %= m[j]                            
+    for i in reversed(range(l)):
+        if m[i] == 1:
+            del m[i]
+            del a[i]
+    l =len(m)
+
+    M = prod(m)
+    Mi = [M // m[i] for i in range(l)]
+    MNi = [Mi[i] * pow(Mi[i], -1, m[i]) % M for i in range(l)]
+    return sum([a[i] * MNi[i] % M for i in range(l)]) % M
