@@ -4,23 +4,31 @@ Polynomials
 
 from numbers import Number
 
+display_reversed = True
+
+def poly_style_reversed(value: bool = True) -> None:
+    "Change the order of the coefficients when displaying polynomials."
+    global display_reversed
+    display_reversed = value
+
+
 class Poly:
     """
     Represents a polynomial as a list of coefficients.
     
     Example:
     
-    To define a polynomial as alist of coefficients use
+    To define a polynomial as a list of coefficients use
     >>> Poly([1, 2, 3])
     3 x^2 + 2 x + 1
     """
 
     def __init__(self, coeff: list, ring = None, modulus: list = None):
-        for i in range(len(coeff) - 1, 0, -1):
-            if coeff[i]:
+        self.coeff = list(coeff)
+        for i in range(len(self.coeff) - 1, 0, -1):
+            if self.coeff[i]:
                 break
-            coeff.pop(i)
-        self.coeff = coeff
+            self.coeff.pop(i)
         self.modulus = modulus
         if ring:
             self.map(ring)
@@ -40,7 +48,7 @@ class Poly:
         return len(self.coeff)
 
     def __repr__(self):
-        def prx(i: int):
+        def prx(i: int) -> str:
             if i == 0:
                 return ""
             if i == 1:
@@ -51,7 +59,10 @@ class Poly:
             return str(self.coeff[0])
         plus = ""
         tmp = ""
-        for i in reversed(range(len(self.coeff))):
+        coef_range = range(len(self.coeff))
+        if display_reversed:
+            coef_range = reversed(coef_range)
+        for i in coef_range:
             s = self.coeff[i]
             if not s:
                 continue
@@ -70,11 +81,15 @@ class Poly:
                 pass
             try:
                 if plus and s < 0:
-                    tmp += " - " + str(-s) + " " + prx(i)
+                    tmp += " - " + str(-s)
+                    if i != 0:
+                        tmp += " " + prx(i)
                     continue
             except:
                 pass
-            tmp += plus + str(s) + " " + prx(i)
+            tmp += plus + str(s)
+            if i != 0:
+                tmp += " " + prx(i)
             plus = " + "
         return tmp.strip()
 
@@ -86,14 +101,18 @@ class Poly:
     def __bool__(self):
         return bool(self.degree()) or bool(self.coeff[0])
 
-    def degree(self):
+    def degree(self) -> int:
         "Return the degree."
         return len(self.coeff) - 1
 
     def map(self, func):
-        "Apply a given function to all coefficients."
+        "Apply a given function to all coefficients in place."
         self.coeff = list(map(func, self.coeff))
 
+    def applyfunc(self, func) -> "Poly":
+        "Apply a function to all coefficients."
+        return self.__class__(list(map(func, self.coeff)), modulus=self.modulus)
+
     def _check_type(self, other):
         return isinstance(other, int) or (isinstance(other, Number) and isinstance(self.coeff[0], Number)) or type(other) == type(self.coeff[0])
 
@@ -129,6 +148,9 @@ class Poly:
     def __neg__(self) -> "Poly":
         return Poly([-s for s in self.coeff], modulus=self.modulus)
 
+    def __pos__(self) -> "Poly":
+        return self
+
     def __sub__(self, other: "Poly") -> "Poly":
         if not isinstance(other, self.__class__):
             if self._check_type(other):
@@ -196,23 +218,36 @@ class Poly:
             raise NotImplementedError("Cannot invert polynomials without modulus.")
         return other * self.inv()
 
-    def __pow__(self, i: int) -> "Poly":
-        if not isinstance(i, int):
+    def __pow__(self, j: int) -> "Poly":
+        if not isinstance(j, int):
             return NotImplemented
         zero = 0 * self.coeff[0]
         one = zero + 1
         res = self.__class__([one], modulus=self.modulus)
-        if i < 0:
+        if j < 0:
             if not self.modulus:
                 raise NotImplementedError("Cannot divide polynomials without modulus.")
             tmp = self.inv()
-            i *= -1
+            j *= -1
         else:
             tmp = self
-        for _ in range(i):
-            res *= tmp
+        while j > 0:
+            # If j is odd, multiply
+            if j & 1:
+                res *= tmp
+            # Now square
+            j = j >> 1  # j= j//2
+            tmp *= tmp
         return res
 
+    def max(self) -> int:
+        "Maximum of the absolute value of all coefficients."
+        return max([abs(c) for c in self.coeff])
+
+    def sum(self) -> int:
+        "Sum of the absolute value of all coefficients."
+        return sum([abs(c) for c in self.coeff])
+
     def __floordiv__(self, other: "Poly") -> "Poly":
         return self.divmod(other)[0]
 
